#include <stdio.h>
#include <stdlib.h>
#include <string.h>

typedef struct
{
    int index;                  /* of input order */
    unsigned weight, iq;
} datum;                        /* a weight-IQ tuple */

datum data[1000], save[1000];   /* stores all input tuples; we prepare
                                   for the maximum, since the input
                                   does not say anything in advance */

int N;                          /* actual number of input tuples */

unsigned ws[1000];              /* workspace for dynamic programming:
                                   ws[i] is the length of the longest
                                   decreasing subsequence starting at
                                   i */
unsigned maxlen;                /* length of the longest decreasing
                                   subsequence */

/* comparison function, so that weights increases, and among equal
   weights, IQ increases too */
int datum_cmp(const void *lhs, const void *rhs)
{
    if ( ((datum*)lhs)->weight == ((datum*)rhs)->weight )
    {
        return ((datum*)lhs)->iq - ((datum*)rhs)->iq;
    }
    else
    {
        return ((datum*)lhs)->weight - ((datum*)rhs)->weight;
    }
}

int main(int argc, char **argv)
{
    int i;                      /* loop counter :) */
    unsigned iq;                /* useful later */
    FILE *ins, *outs;           /* official input and submitted output */
    unsigned usermaxlen, userweight, useriq;
    char s[100];                /* to hold an input line */

    /* open files */
    ins = fopen(argv[1], "r");
    outs = fopen(argv[2], "r"); /* yes, "r", since we are checking it */

    /* read input */
    N = 0;
    while (fgets(s, 100, ins) != NULL)
    {
        sscanf(s, "%u%u", &data[N].weight, &data[N].iq);
        data[N].index = N + 1;
        N++;
    }

    /* save a copy of the original data, in the original order */
    memcpy(save, data, sizeof(datum) * N);

    /* sort by increasing weight, with a catch: for equal weights,
       sort by increasing IQ, so that the answer contains distinct
       weights */
    qsort(data, N, sizeof(datum), datum_cmp);

    /* now look for the longest strictly decreasing subsequence in IQs
       by dynamic programming */
    maxlen = 0;
    for (i = N-1; i >= 0; i--)
    {
        int j;
        ws[i] = 1;
        /* see if i can extend known subsequences */
        for (j = i+1; j < N; j++)
        {
            if (data[i].iq > data[j].iq && ws[j] + 1 > ws[i])
                ws[i] = ws[j] + 1;
        }
        if (ws[i] > maxlen) maxlen = ws[i];
    }

    /* now we know the correct answer to maxlen; proceed to check
       contestant's output */
    /* note the robustness: any reading error, format error, out-of-range
       error, or what-have-you is caught */
    if (fgets(s, 100, outs) == NULL){
	printf( "None\n" );
	return 1;
    }
    if (sscanf(s, "%u", &usermaxlen) != 1){
	printf( "No int\n" );
	return 1;
    }
    if (usermaxlen != maxlen) return 1;
    userweight = 0;             /* effectively -inf */
    useriq = 10001;             /* effectively +inf */
    for (i = 0; i < maxlen; i++)
    {
        int k;
        if (fgets(s, 100, outs) == NULL){
	    printf( "Too few\n" );
	    return 1;
	}
        if (sscanf(s, "%d", &k) != 1){
	    printf( "No int\n" );
	    return 1;
	}

        if (k < 1 || k > N){
	    printf( "int out of range\n" );
	    return 1;
	}

        /* of course, we catch incorrect solutions too */
        if (userweight >= save[k-1].weight){
	    printf( "weight out of order\n" );
	    return 1;
	}
        if (useriq <= save[k-1].iq){
	    printf( "IQ out of order\n" );
	    return 1;
	}
        userweight = save[k-1].weight;
        useriq = save[k-1].iq;
    }
    /* if we get this far, aok */
    return 0;
}
